from strawberry.dataloader import DataLoader

from bookshelves.models import Book


async def load_books(
    category_ids: list[int],
) -> list[list[Book]]:
    """Bookのデータローダに渡すバッチ関数"""
    # SQLを発行して、
    # CategoryとBookを結合した結果をIN句で一括取得する
    books = [
        book
        async for book in Book.objects.filter(
            category_id__in=category_ids
        ).select_related("category")
    ]
    # カテゴリIDがキーで対応する本(複数)が
    # 値の辞書を作成する
    category_id_to_books_dict: dict[int, list[Book]] = {
        category_id: [] for category_id in category_ids
    }
    for book in books:
        category_id_to_books_dict[
            book.category.id
        ].append(book)
    # 作成した辞書を使って
    # 引数のID順に揃えたリストを作成して返す
    return [
        category_id_to_books_dict[category_id]
        for category_id in category_ids
    ]


# Bookのデータローダ
books_loader = DataLoader(load_fn=load_books)
